@@ -1,13 +1,14 @@
 # -*- coding: utf-8 -*-
 import asyncio
-import json
 import copy
+import inspect
+import json
 from collections import namedtuple
-from distutils.version import StrictVersion
 from functools import wraps
 from typing import Callable, Dict, Tuple, Union, Optional, List  # noqa
 from unittest.mock import Mock, patch
-import inspect
+from uuid import uuid4
+
 from aiohttp import (
     ClientConnectionError,
     ClientResponse,
@@ -17,6 +18,7 @@ from aiohttp import (
 )
 from aiohttp.helpers import TimerNoop
 from multidict import CIMultiDict, CIMultiDictProxy
+from pkg_resources import parse_version
 
 from .compat import (
     AIOHTTP_VERSION,
@@ -130,7 +132,7 @@ class RequestMatch(object):
         if request_headers is None:
             request_headers = {}
         kwargs = {}
-        if AIOHTTP_VERSION >= StrictVersion('3.1.0'):
+        if AIOHTTP_VERSION >= parse_version('3.1.0'):
             loop = Mock()
             loop.get_debug = Mock()
             loop.get_debug.return_value = True
@@ -142,7 +144,7 @@ class RequestMatch(object):
             kwargs['writer'] = Mock()
             kwargs['continue100'] = None
             kwargs['timer'] = TimerNoop()
-            if AIOHTTP_VERSION < StrictVersion('3.3.0'):
+            if AIOHTTP_VERSION < parse_version('3.3.0'):
                 kwargs['auto_decompress'] = True
             kwargs['traces'] = []
             kwargs['loop'] = loop
@@ -159,7 +161,7 @@ class RequestMatch(object):
         for hdr in _headers.getall(hdrs.SET_COOKIE, ()):
             resp.cookies.load(hdr)
 
-        if AIOHTTP_VERSION >= StrictVersion('3.3.0'):
+        if AIOHTTP_VERSION >= parse_version('3.3.0'):
             # Reified attributes
             resp._headers = _headers
             resp._raw_headers = raw_headers
@@ -174,7 +176,7 @@ class RequestMatch(object):
         return resp
 
     async def build_response(
-            self, url: URL, **kwargs
+        self, url: URL, **kwargs
     ) -> 'Union[ClientResponse, Exception]':
         if self.exception is not None:
             return self.exception
@@ -206,7 +208,7 @@ RequestCall = namedtuple('RequestCall', ['args', 'kwargs'])
 
 class aioresponses(object):
     """Mock aiohttp requests made by ClientSession."""
-    _matches = None  # type: List[RequestMatch]
+    _matches = None  # type: Dict[str, RequestMatch]
     _responses = None  # type: List[ClientResponse]
     requests = None  # type: Dict
 
@@ -253,7 +255,7 @@ class aioresponses(object):
 
     def start(self):
         self._responses = []
-        self._matches = []
+        self._matches = {}
         self.patcher.start()
         self.patcher.return_value = self._request_mock
 
@@ -296,7 +298,8 @@ class aioresponses(object):
             timeout: bool = False,
             reason: Optional[str] = None,
             callback: Optional[Callable] = None) -> None:
-        self._matches.append(RequestMatch(
+
+        self._matches[str(uuid4())] = (RequestMatch(
             url,
             method=method,
             status=status,
@@ -324,12 +327,12 @@ class aioresponses(object):
         return False
 
     async def match(
-            self, method: str, url: URL,
-            allow_redirects: bool = True, **kwargs: Dict
+        self, method: str, url: URL,
+        allow_redirects: bool = True, **kwargs: Dict
     ) -> Optional['ClientResponse']:
         history = []
         while True:
-            for i, matcher in enumerate(self._matches):
+            for key, matcher in self._matches.items():
                 if matcher.match(method, url):
                     response_or_exc = await matcher.build_response(
                         url, allow_redirects=allow_redirects, **kwargs
@@ -339,13 +342,12 @@ class aioresponses(object):
                 return None
 
             if matcher.repeat is False:
-                del self._matches[i]
+                del self._matches[key]
 
             if self.is_exception(response_or_exc):
                 raise response_or_exc
-
-            if response_or_exc.status in (
-                    301, 302, 303, 307, 308) and allow_redirects:
+            is_redirect = response_or_exc.status in (301, 302, 303, 307, 308)
+            if is_redirect and allow_redirects:
                 if hdrs.LOCATION not in response_or_exc.headers:
                     break
                 history.append(response_or_exc)
@@ -380,7 +382,7 @@ class aioresponses(object):
         self.requests.setdefault(key, [])
         try:
             kwargs_copy = copy.deepcopy(kwargs)
-        except TypeError:
+        except (TypeError, ValueError):
             # Handle the fact that some values cannot be deep copied
             kwargs_copy = kwargs
         self.requests[key].append(RequestCall(args, kwargs_copy))
